#
# Copyright © 2022-2024 University of Strasbourg. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from mimiqlink.abstractconnection import (
    AbstractConnection,
)

from mimiqlink.mimiqconnection import (
    MimiqConnection,
    QPERFECT_CLOUD,
    QPERFECT_DEV,
)

from mimiqlink.planqkconnection import (
    PlanqkConnection,
    PLANQK_API,
)

__all__ = [
    "AbstractConnection",
    "MimiqConnection",
    "PlanqkConnection",
    "QPERFECT_CLOUD",
    "QPERFECT_DEV",
    "PLANQK_API",
]
